export type FontProps = {
  fontSize?: number;
  fontFamily?: string;
  fontStyle?: 'normal' | 'italic';
  fontWeight?:
    | 'normal'
    | 'bold'
    | 100
    | 200
    | 300
    | 400
    | 500
    | 600
    | 700
    | 800
    | 900;
};

export type TextStyleProps = FontProps & {
  color?: string;
  lineHeight?: number;
  textAlign?: 'auto' | 'left' | 'right' | 'center' | 'justify';
  textAlignVertical?: 'auto' | 'top' | 'bottom' | 'center';
  letterSpacing?: number;
  textTransform?: 'none' | 'uppercase' | 'lowercase' | 'capitalize';
};

export type ItemProps = {
  width?: number;
  height?: number;
  x?: number;
  y?: number;
  z?: number;
  implicitWidth?: number;
  implicitHeight?: number;
  opacity?: number;
  visible?: boolean;
  clip?: boolean;
  antialiasing?: boolean;
  baselineOffset?: number;
};

export type TransformProps = {
  scale?: number;
  rotation?: number;
};

export type Style = TextStyleProps &
  ItemProps &
  TransformProps & { [key: string]: any };

const FONT_WEIGHT_MAP: { [key: string]: number } = {
  normal: 50,
  bold: 700,
  '100': 0,
  '200': 12,
  '300': 25,
  '400': 50,
  '500': 57,
  '600': 63,
  '700': 75,
  '800': 81,
  '900': 87,
};
function getFontWeight(value: string) {
  if (Object.keys(FONT_WEIGHT_MAP).indexOf(value) > -1) {
    return FONT_WEIGHT_MAP[value];
  }
  return 400;
}

const FONT_CAPITALIZATION_MAP: { [key: string]: string } = {
  none: 'MixedCase',
  uppercase: 'AllUppercase',
  lowercase: 'AllLowercase',
  capitalize: 'Capitalize',
};
function getFontCapitalization(value: string) {
  if (Object.keys(FONT_CAPITALIZATION_MAP).indexOf(value) > -1) {
    return FONT_CAPITALIZATION_MAP[value];
  }
  return 'MixedCase';
}

const TEXT_ALIGN_MAP: { [key: string]: string } = {
  left: 'AlignLeft',
  right: 'AlignRight',
  center: 'AlignHCenter',
  justify: 'AlignJustify',
};
function getTextAlignment(value: string) {
  if (Object.keys(TEXT_ALIGN_MAP).indexOf(value) > -1) {
    return TEXT_ALIGN_MAP[value];
  }
  return undefined;
}

const TEXT_ALIGN_VERTICAL_MAP: { [key: string]: string } = {
  top: 'AlignTop',
  bottom: 'AlignBottom',
  center: 'AlignVCenter',
};
function getTextAlignmentVertical(value: string) {
  if (Object.keys(TEXT_ALIGN_VERTICAL_MAP).indexOf(value) > -1) {
    return TEXT_ALIGN_VERTICAL_MAP[value];
  }
  return undefined;
}

export function mapStyleToProps(style: Style): Qml.QmlProps {
  // no validation, for now
  let props: any = {};
  let font: Qml.QmlFontProps = {};
  let Layout = {};
  for (const styleName in style) {
    const styleValue = style[styleName];
    switch (styleName) {
      case 'fontSize':
        font.pointSize = styleValue;
        break;
      case 'fontFamily':
        font.family = styleValue;
        break;
      case 'fontStyle':
        if (styleValue === 'italic') {
          font.italic = true;
        }
        break;
      case 'fontWeight':
        font.weight = getFontWeight(String(styleValue));
        break;
      case 'textAlign':
        props.horizontalAlignment = getTextAlignment(String(styleValue));
        break;
      case 'textAlignVertical':
        props.verticalAlignment = getTextAlignmentVertical(String(styleValue));
        break;
      case 'textTransform':
        font.capitalization = getFontCapitalization(String(styleValue));
        break;

      default:
        // passthrough
        props[styleName] = styleValue;
        break;
    }
  }

  return {
    ...props,
    Layout,
    font,
  };
}

const isEmpty = (obj: object) => Object.keys(obj).length === 0 && obj.constructor === Object

export function setStyle(element: Qml.QmlElement, style: Style) {
  const { Layout, font, ...props } = mapStyleToProps(style);
  if () {
    
  }
}
