"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _invariant = _interopRequireDefault(require("invariant"));

var _qml = require("./qml");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var rootContext = {
  element: _qml.hostElement,
  metadata: {
    defaultProp: 'data'
  }
};
var childContext = rootContext;
var hostConfig = {
  now: Date.now,
  isPrimaryRenderer: true,
  noTimeout: -1,
  supportsPersistence: false,
  supportsHydration: false,
  shouldDeprioritizeSubtree: function shouldDeprioritizeSubtree(type, props) {
    return false;
  },
  scheduleDeferredCallback: function scheduleDeferredCallback(callback, options) {},
  cancelDeferredCallback: function cancelDeferredCallback(callbackID) {},
  setTimeout: function setTimeout(handler, timeout) {
    if (global && global.setTimeout) {
      return global.setTimeout(handler, timeout);
    }

    return -1;
  },
  clearTimeout: function clearTimeout(handle) {
    if (typeof handle !== 'number' && global && global.clearTimeout) {
      global.clearTimeout(handle);
    }
  },
  // general
  getPublicInstance: function getPublicInstance(instance) {
    // return qml element
    return instance.element;
  },
  getRootHostContext: function getRootHostContext(rootContainerInstance) {
    var rootElement = rootContext.element;
    console.log('getRootHostContext', rootContext.element);
    (0, _qml.removeAllChildren)(rootElement, rootContext.metadata.defaultProp);
    return rootContext;
  },
  getChildHostContext: function getChildHostContext(parentHostContext, type, rootContainerInstance) {
    console.log('getChildHostContext', parentHostContext.element, type, rootContainerInstance);
    return childContext;
  },
  shouldSetTextContent: function shouldSetTextContent(type, props) {
    console.log('shouldSetTextContent', type, props);
    return false;
  },
  createTextInstance: function createTextInstance(text, rootContainerInstance, hostContext, internalInstanceHandle) {
    console.log('createTextInstance', text, rootContainerInstance, hostContext.element, internalInstanceHandle);
    (0, _invariant.default)(false, 'Creating text instance not supported');
    var nullElement = {
      parent: null,
      destroy: function destroy() {}
    };
    return {
      element: nullElement,
      metadata: {
        defaultProp: 'data'
      }
    };
  },
  createInstance: function createInstance(type, props, rootContainerInstance, hostContext, internalInstanceHandle) {
    console.log('createInstance', type, props, rootContainerInstance, hostContext.element, internalInstanceHandle);
    return (0, _qml.createElementContainer)(type, props, rootContainerInstance, hostContext);
  },
  appendInitialChild: function appendInitialChild(parent, child) {
    console.log('appendInitialChild', parent.element, child.element);
    (0, _qml.appendChild)(parent, child);
  },
  finalizeInitialChildren: function finalizeInitialChildren(parentInstance, type, props, rootContainerInstance, hostContext) {
    console.log('finalizeInitialChildren', parentInstance.element, type, props, rootContainerInstance, hostContext.element);
    return false;
  },
  prepareForCommit: function prepareForCommit(rootContainerInstance) {
    console.log('prepareForCommit', rootContainerInstance);
  },
  resetAfterCommit: function resetAfterCommit(rootContainerInstance) {
    console.log('resetAfterCommit', rootContainerInstance);
  },
  // -------------------
  //     Mutation
  // -------------------
  supportsMutation: true,
  appendChild: function appendChild(parent, child) {
    console.log('appendChild', parent.element, child.element);
    (0, _qml.appendChild)(parent, child);
  },
  appendChildToContainer: function appendChildToContainer(container, child) {
    console.log('appendChildToContainer', container, child.element);
    (0, _qml.appendChildToContainer)(container, child);
  },
  commitTextUpdate: function commitTextUpdate(textInstance, oldText, newText) {
    (0, _invariant.default)(false, 'Text instance not supported');
  },
  resetTextContent: function resetTextContent(instance) {
    (0, _invariant.default)(false, 'Text instance not supported');
  },
  commitMount: function commitMount(instance, type, newProps, fiberNode) {
    console.log('commitMount', instance, type, newProps, fiberNode);
  },
  prepareUpdate: function prepareUpdate(instance, type, oldProps, newProps, rootContainerInstance, hostContext) {
    console.log('prepareUpdate');
    return {};
  },
  commitUpdate: function commitUpdate(instance, updatePayloadTODO, type, oldProps, newProps, internalInstanceHandle) {
    console.log('commitUpdate');
    var updatePayload = (0, _qml.diffProps)(instance.element, oldProps, newProps);

    if (updatePayload != null) {
      // update props
      (0, _qml.updateProps)(instance.element, updatePayload);
    }
  },
  removeChild: function removeChild(parent, child) {
    console.log('removeChild', parent.element, child.element);
    (0, _qml.removeChild)(parent, child);
  },
  removeChildFromContainer: function removeChildFromContainer(container, child) {
    console.log('removeChildFromContainer', container, child.element);
    (0, _qml.removeChildFromContainer)(container, child);
  },
  insertBefore: function insertBefore(parentInstance, child, beforeChild) {
    (0, _qml.insertBefore)(parentInstance, child, beforeChild);
  },
  insertInContainerBefore: function insertInContainerBefore(container, child, beforeChild) {
    (0, _qml.insertInContainerBefore)(container, child, beforeChild);
  }
};
var _default = hostConfig;
exports.default = _default;