"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.diffProps = diffProps;
exports.updateProps = updateProps;
exports.createElementContainer = createElementContainer;
exports.appendChild = appendChild;
exports.appendChildToContainer = appendChildToContainer;
exports.removeChild = removeChild;
exports.removeChildFromContainer = removeChildFromContainer;
exports.removeAllChildren = removeAllChildren;
exports.insertBefore = insertBefore;
exports.insertInContainerBefore = insertInContainerBefore;
exports.hostElement = void 0;

var _util = require("util");

var _Anchor = require("../common/Anchor");

var _QmlElementContainer = _interopRequireDefault(require("./QmlElementContainer"));

var _AppRegistry = _interopRequireDefault(require("../common/AppRegistry"));

var _StyleSheet = _interopRequireDefault(require("../common/StyleSheet"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance"); }

function _iterableToArrayLimit(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

function _typeof(obj) { if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

// QML signal handler convention
var qmlSignalRegex = /^on([A-Z][a-zA-Z]+)$/;

function listenTo(qmlElement, eventName, nextHandler, lastHandler) {
  eventName = eventName[0].toLowerCase() + eventName.substring(1);
  console.log('listenTo', qmlElement, eventName, nextHandler);

  if (!qmlElement[eventName]) {
    console.warn("Event \"".concat(eventName, "\" not found in ").concat(qmlElement));
    return;
  }

  if (lastHandler) {
    qmlElement[eventName].disconnect(lastHandler);
  }

  if (nextHandler) {
    qmlElement[eventName].connect(nextHandler);
  }
} // handle connections


function handleAnchors(qmlElement, lastAnchors, nextAnchors) {
  // last anchors
  for (var propName in lastAnchors) {
    if (lastAnchors.hasOwnProperty(propName)) {
      if ((0, _Anchor.isAnchorProp)(propName)) {
        var anchorRef = lastAnchors[propName];

        if (typeof anchorRef === 'string') {
          _Anchor.ParentAnchor.removeSubscription(qmlElement, propName);

          qmlElement.anchors[propName] = undefined;
          continue;
        }

        if (anchorRef) {
          anchorRef.removeSubscription(qmlElement, propName);
        } // unset anchor


        qmlElement.anchors[propName] = undefined;
      }
    }
  } // next anchors


  for (var _propName in nextAnchors) {
    if (nextAnchors.hasOwnProperty(_propName)) {
      if ((0, _Anchor.isAnchorProp)(_propName)) {
        var _anchorRef = nextAnchors[_propName];

        if (typeof _anchorRef === 'string') {
          _Anchor.ParentAnchor.addSubscription(qmlElement, _propName, _anchorRef);

          continue;
        }

        if (_anchorRef) {
          // anchor, set value when anchor ready
          _anchorRef.addSubscription(qmlElement, _propName);
        } else {
          // unset anchor
          qmlElement.anchors[_propName] = undefined;
        }
      } else {
        // primitive, set values right away
        var propValue = nextAnchors[_propName];
        qmlElement.anchors[_propName] = propValue;
      }
    }
  }
} // handle anchor ref


function handleAnchorRef(qmlElement, lastRef, nextRef) {
  // TODO: handle last ref
  if (nextRef) {
    nextRef.setQmlElement(qmlElement);
  }
}

function handleStyle(qmlElement, lastStyle, nextStyle) {
  // last style
  var nextFlatStyle = _StyleSheet.default.flattenStyle(nextStyle);

  var finalStyle = nextFlatStyle;

  if (lastStyle) {
    var lastFlatStyle = _StyleSheet.default.flattenStyle(lastStyle);

    for (var styleName in lastFlatStyle) {
      if (!nextFlatStyle.hasOwnProperty(styleName)) {
        // unset
        finalStyle[styleName] = undefined;
      }
    }
  }

  _StyleSheet.default.setStyle(qmlElement, finalStyle);
}

function setInitialProps(qmlElement, props) {
  for (var propKey in props) {
    var propValue = props[propKey];

    if (propKey === 'children' || propValue == null) {
      // ignore
      continue;
    } // event handling


    var matches = propKey.match(qmlSignalRegex);

    if (matches) {
      var eventName = matches[1];
      listenTo(qmlElement, eventName, propValue, null);
      continue;
    } // anchor ref


    if (propKey === 'anchorRef') {
      handleAnchorRef(qmlElement, null, propValue);
      continue;
    } // anchors handling


    if (propKey === 'anchors') {
      handleAnchors(qmlElement, null, propValue);
      continue;
    } // style handling


    if (propKey === 'style') {
      handleStyle(qmlElement, null, propValue);
      continue;
    } // attached property


    if (_typeof(propValue) === 'object' && _typeof(qmlElement[propKey]) === 'object') {
      console.log('setInitialProps', propKey);
      console.log((0, _util.inspect)(propValue, {
        depth: 1
      }));

      if (qmlElement[propKey]) {
        Object.assign(qmlElement[propKey], propValue);
      }

      continue;
    }

    if (!qmlElement.hasOwnProperty(propKey)) {
      console.warn("Cannot assign to non-existent property \"".concat(propKey, "\""));
      continue;
    }

    qmlElement[propKey] = propValue;
  }
} // Calculate the diff between the two objects.


function diffProps(qmlElement, lastProps, nextProps) {
  var updatePayload = []; // phase 1: look for deleted props

  for (var propKey in lastProps) {
    if (lastProps[propKey] == null || !lastProps.hasOwnProperty(propKey) || nextProps.hasOwnProperty(propKey)) {
      continue;
    }

    updatePayload.push(propKey, null);
  } // phase 2: look for actual changes


  for (var _propKey in nextProps) {
    var nextProp = nextProps[_propKey];
    var lastProp = lastProps != null ? lastProps[_propKey] : undefined;

    if (!nextProps.hasOwnProperty(_propKey) || nextProp === lastProp || nextProp == null && lastProp == null) {
      continue;
    } // event handling


    var matches = _propKey.match(qmlSignalRegex);

    var isEventProp = matches && lastProp !== nextProp;
    var isAnchorsProp = _propKey === 'anchors';
    var isAnchorRefProp = _propKey === 'anchorRef';
    var isStyleProp = _propKey === 'style';

    if (isEventProp || isAnchorsProp || isAnchorRefProp || isStyleProp) {
      updatePayload.push(_propKey, [lastProp, nextProp]);
      continue;
    }

    updatePayload.push(_propKey, nextProp);
  }

  return updatePayload.length === 0 ? null : updatePayload;
} // Apply the diff.


function updateProps(qmlElement, updatePayload) {
  for (var i = 0; i < updatePayload.length; i += 2) {
    var propKey = updatePayload[i];
    var propValue = updatePayload[i + 1]; // ignore the children

    if (propKey === 'children') {
      continue;
    } // event handling


    var matches = propKey.match(qmlSignalRegex);

    if (matches) {
      var eventName = matches[1];

      var _propValue = _slicedToArray(propValue, 2),
          lastHandler = _propValue[0],
          nextHandler = _propValue[1];

      listenTo(qmlElement, eventName, nextHandler, lastHandler);
      continue;
    } // anchor ref


    if (propKey === 'anchorRef') {
      var _propValue2 = _slicedToArray(propValue, 2),
          lastRef = _propValue2[0],
          nextRef = _propValue2[1];

      handleAnchorRef(qmlElement, lastRef, nextRef);
      continue;
    } // anchors handling


    if (propKey === 'anchors') {
      var _propValue3 = _slicedToArray(propValue, 2),
          lastAnchors = _propValue3[0],
          nextAnchors = _propValue3[1];

      handleAnchors(qmlElement, lastAnchors, nextAnchors);
      continue;
    } // style handling


    if (propKey === 'style') {
      var _propValue4 = _slicedToArray(propValue, 2),
          lastStyle = _propValue4[0],
          nextStyle = _propValue4[1];

      handleStyle(qmlElement, lastStyle, nextStyle);
      continue;
    } // attached property


    if (_typeof(propValue) === 'object' && _typeof(qmlElement[propKey]) === 'object') {
      console.log('updateProps', propKey);
      console.log((0, _util.inspect)(propValue, {
        depth: 1
      }));

      if (qmlElement[propKey]) {
        Object.assign(qmlElement[propKey], propValue);
      }

      continue;
    }

    if (!qmlElement.hasOwnProperty(propKey)) {
      console.warn("Cannot assign to non-existent property \"".concat(propKey, "\""));
      continue;
    }

    qmlElement[propKey] = propValue;
  }
} // we need a TempRoot when rendering React tree
// the old method (create element as direct child of rootContainerInstance)
// is not working when error happended


function createHostContext() {
  var qml = "import QtQuick 2.7; Item { visible: false; }";
  return Qt.createQmlObject(qml, Qt.application, 'HostContext');
}

var hostElement = createHostContext(); // Create new QmlElementContainer

exports.hostElement = hostElement;

function createElementContainer(type, props, rootContainerInstance, hostContext) {
  var hostElement = hostContext.element;

  var componentDefinition = _AppRegistry.default.getComponent(type);

  if (componentDefinition) {
    var component = componentDefinition.component,
        metadata = componentDefinition.metadata;
    var element = component.createObject(hostElement, props);

    if (!element) {
      throw new Error("Unable to create element: ".concat(type));
    }

    setInitialProps(element, props);
    console.log('\n\n\nCreate new element');
    console.log(element, '\n\n\n');
    return new _QmlElementContainer.default(element, metadata);
  } // fall back to raw components


  var rawComponentDefinition = _AppRegistry.default.getRawComponent(type);

  if (rawComponentDefinition) {
    var rawContent = rawComponentDefinition.rawContent,
        _metadata = rawComponentDefinition.metadata;

    var _element = Qt.createQmlObject(rawContent, hostElement, type);

    setInitialProps(_element, props);
    console.log('\n\n\nCreate new element');
    console.log(_element, '\n\n\n');
    return new _QmlElementContainer.default(_element, _metadata);
  }

  throw new Error("Unknown type ".concat(type));
} // TODO: revise this later


function isQuickItem(obj) {
  var isQtObject = Qt.isQtObject(obj);
  var hasChildAtMethod = typeof obj.childAt === 'function';
  var hasParentProp = obj.hasOwnProperty('parent');
  var hasDataProp = obj.hasOwnProperty('data');
  return isQtObject && hasChildAtMethod && hasParentProp && hasDataProp;
}

function isWindow(obj) {
  var isQtObject = Qt.isQtObject(obj);
  var hasWindowMethods = typeof obj.showMinimized === 'function' && typeof obj.showMaximized === 'function' && typeof obj.showFullScreen === 'function';
  return isQtObject && hasWindowMethods;
}

function getObjectType(obj) {
  if (!Qt.isQtObject(obj)) {
    return '';
  } // debug name is in the form of QObjectType(memory_address);
  // we need to remove the memory address part


  var debugName = obj.toString();
  return debugName.replace(/\(.+\)/, '');
}

function canSetParent(obj) {
  var isQtObject = Qt.isQtObject(obj);
  var hasParentProp = obj.hasOwnProperty('parent');
  return isQtObject && hasParentProp;
}

function removeElementFromHostContext(element) {
  var hostData = hostElement.data;

  if (hostData && hostData.indexOf) {
    var childIndex = hostData.indexOf(element);

    if (childIndex > -1) {
      hostData.splice(childIndex, 1);
    }
  }
}

function isAnimation(obj) {
  var isQtObject = Qt.isQtObject(obj);
  var objType = getObjectType(obj);
  var isAnimationType = objType === 'QQuickColorAnimation' || objType === 'QQuickNumberAnimation' || objType === 'QQuickPropertyAnimation' || objType === 'Vector3dAnimation' || objType === 'RotationAnimation' || objType === 'AnchorAnimation' || objType === 'ParentAnimation ' || objType === 'QQuickSmoothedAnimation' || objType === 'QQuickSpringAnimation';
  return isQtObject && isAnimationType;
} // turned out, appending child in qml is not as simple as setting prop `.parent`
// - if both parent and child are Item, simply setting parent would do
// - otherwise, we can append child to parent's default prop (eg: parent.data)
//   in special cases we don't need to do anything (eg: child is an instance of Window)


function appendChildElement(parent, child) {
  var parentDefaultProp = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 'data';
  removeElementFromHostContext(child);
  var parentType = getObjectType(parent);
  var childType = getObjectType(child); // special cases
  // - add MenuBar to ApplicationWindow

  if (childType === 'QQuickMenuBar' && parentType === 'QQuickApplicationWindow') {
    parent.menuBar = child;
    return;
  } // - sync QQuickPlatformMenu


  if (childType === 'QQuickPlatformMenu' && parentType === 'QQuickPlatformMenuBar') {
    parent.addMenu(child); // this is a hack to "sync" menu

    child.visible = false;
    child.visible = true;
    return;
  } // - set animation's target to parent, if not specified


  if (isAnimation(child) && !isAnimation(parent)) {
    child.target = parent;
  }

  if (isQuickItem(parent) && isQuickItem(child)) {
    console.log('child.parent=');
    child.parent = parent;
    return;
  } // append child to parent's default prop


  var parentData = parent[parentDefaultProp];

  if (parentData && parentData.push) {
    parentData.push(child);
  }
}

function appendChild(parentContainer, childContainer) {
  var parent = parentContainer.element;
  var child = childContainer.element;
  appendChildElement(parent, child, parentContainer.metadata.defaultProp);
}

function appendChildToContainer(container, childContainer) {
  var child = childContainer.element;
  appendChildElement(container, child);
}

function removeChildElement(parent, child) {
  var parentDefaultProp = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 'data';
  var parentType = getObjectType(parent);
  var childType = getObjectType(child);
  console.log('parentType', parentType);
  console.log('childType', childType);

  if (isQuickItem(parent) && isQuickItem(child)) {
    child.parent = null;
    console.log('removeChildElement', parent, child);
  } else {
    console.log('herereree');
    var parentData = parent[parentDefaultProp];

    if (parentData) {
      var childIndex = findChildIndex(parentData, child);

      if (childIndex > -1) {
        if (parent.remove) {
          parent.remove(childIndex);
          console.log('removeChildElement', parent, 'children length = ', parentData.length);
        } else {
          console.log('bo tay'); // parent[parentDefaultProp].splice(childIndex, 1);
        }
      }
    }
  }

  child.destroy();
}

function removeChild(parentContainer, childContainer) {
  var parent = parentContainer.element;
  var child = childContainer.element;
  removeChildElement(parent, child, parentContainer.metadata.defaultProp);
}

function removeChildFromContainer(container, childContainer) {
  removeChildElement(container, childContainer.element);
}

function removeAllChildren(parent) {
  var defaultProp = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 'data';
  var data = parent[defaultProp];
  console.log('removeAllChildren', parent, 'children length = ', data.length);

  if (data && data.length > 0) {
    // destroy all children
    for (var index = data.length; index > 0; index--) {
      var child = data[index - 1];
      removeChildElement(parent, child);
    }

    console.log('children no = ', data.length);
  }
}

function findChildIndex(parentData, child) {
  for (var index = 0; index < parentData.length; index++) {
    var element = parentData[index];

    if (element === child) {
      return index;
    }
  }

  return -1;
}

function moveChild(parentData, fromIndex, toIndex) {
  if (fromIndex < 0 || fromIndex >= parentData.length) {
    throw new Error('Out of range: fromIndex');
  }

  if (toIndex < 0 || toIndex >= parentData.length) {
    throw new Error('Out of range: toIndex');
  }

  var child = parentData[fromIndex];

  if (fromIndex < toIndex) {
    var index = fromIndex;

    while (index < toIndex) {
      parentData[index] = parentData[index + 1];
      index++;
    }

    parentData[toIndex] = child;
  } else {
    var _index = fromIndex;

    while (_index > toIndex) {
      parentData[_index] = parentData[_index - 1];
      _index--;
    }

    parentData[toIndex] = child;
  }

  parentData[toIndex].z = 10;
}

function insertBeforeElement(parent, child, beforeChild) {
  var parentDefaultProp = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : 'data';
  var parentData = parent[parentDefaultProp];

  if (parentData) {
    // there is no indexOf, sad
    // @see https://stackoverflow.com/questions/43030433/qtquick-item-children-indexof-doesnt-exist
    var childIndex = findChildIndex(parentData, child);
    var beforeChildIndex = findChildIndex(parentData, beforeChild); // Move existing child or add new child?

    if (childIndex >= 0) {// move from childIndex to beforeChildIndex
      // moveChild(parentData, childIndex, beforeChildIndex);
    } else {
      // insert child into beforeChildIndex first
      appendChildElement(parent, child, parentDefaultProp); // then move
      // moveChild(parentData, parentData.length - 1, beforeChildIndex);
    }
  }
}

function insertBefore(parentContainer, childContainer, beforeChildContainer) {
  var parent = parentContainer.element;
  var child = childContainer.element;
  var beforeChild = beforeChildContainer.element;
  insertBeforeElement(parent, child, beforeChild, parentContainer.metadata.defaultProp);
}

function insertInContainerBefore(container, childContainer, beforeChildContainer) {
  var child = childContainer.element;
  var beforeChild = beforeChildContainer.element;
  insertBeforeElement(container, child, beforeChild);
}